import { exec } from "shelljs";
import { NodeSSH } from "node-ssh";

require("console-info");
require("console-warn");
require("console-error");

const execOptions = { encoding: "utf-8", silent: true };

async function lintCheck() {
  return new Promise((resolve, reject) => {
    exec("npm run lint", execOptions, (code, stdout, stderr) => {
      if (code === 0) {
        console.info("Lint ok. You've written some clean code! Well Done!");
        resolve(true);
      } else {
        console.error(
          "Lint failed. Your code is a mess, I won't deploy, fix it first."
        );
        resolve(false);
      }
    });
  });
}

async function gitCheck() {
  return new Promise((resolve, reject) => {
    exec("git status --porcelain", execOptions, (code, stdout, stderr) => {
      if (code === 0 && stderr.trim() === "" && stdout.trim() === "") {
        console.info("Git repository is clean.");
        resolve(true);
      } else {
        console.error("Git repository is not clean! Please clean it first.");
        console.log(stderr);
        resolve(false);
      }
    });
  });
}

async function tscBuildCheck() {
  return new Promise((resolve, reject) => {
    exec("npm run tsc", execOptions, (code, stdout, stderr) => {
      if (code === 0) {
        console.info("Building project was successful.");
        resolve(true);
      } else {
        console.error(
          "Failed to build project via typscript. Please fix bugs."
        );
        console.log(stdout);
        console.log(stderr);
        resolve(false);
      }
    });
  });
}

/* 2 dolgot lehet tenni.
 ** 1., local gépen build, a kész index.js és .env fájlokat felmásolni.
 ** 2., ssh-> linux-deploy-t meghívjuk... ami mindent megcsinál, de ha változik a .env, akkor kézzel javítani... szerintem ez legyen paraméterezhető!, viszont akkor a prod.env hová kerül?
 */
async function sshConnect(config) {
  const ssh = new NodeSSH();
  const conn = await ssh.connect(config.sshOptions);
  if (conn instanceof Error) {
    console.error(conn);
    return false;
  }
  console.info(
    `Connected to ${config.sshOptions.host} as ${config.sshOptions.username}`
  );

  for (let cmd of config.remoteCommands) {
    // console.log(cmd);
    const res = await conn.exec(cmd.cmd, cmd.parameters, {
      cwd: cmd.cwd || "./",
    });
    if (res instanceof Error) {
      console.error(res);
      return false;
    } else {
      console.log(res);
    }
    console.info(
      cmd.cmd + " " + cmd.parameters.join(" ") + " command finished!"
    );
  }
  return true;
}

export async function run(options) {
  if (!options.gitSkip) {
    if (!(await gitCheck())) process.exit();
  } else {
    console.info("Git repository check skipped!");
  }
  if (!options.lintSkip) {
    if (!(await lintCheck())) process.exit();
  } else {
    console.info("Lint checking skipped!");
  }

  const tscStatus = await tscBuildCheck();
  if (!tscStatus) process.exit();
  if (!(await sshConnect(options.config))) process.exit();
  console.info("Done.");
  process.exit(0);
}
